/**
 * Mode plugin base.
 *
 * A base class for Navigation+ mode plugins.
 */
export class ModePluginBase {

  /**
   * Initialize the plugin.
   */
  init() {}

  /**
   * Enable the mode.
   *
   * @returns {Promise}
   */
  enable() {
    const topBar = document.querySelector('#' + this.id + '-top-bar');
    if (topBar) {
      topBar.classList.remove('navigation-plus-hidden');
    }
    return Promise.resolve();
  }

  /**
   * Disable the mode.
   *
   * @param modesDisabled
   *   False if we are just disabling the mode to switch to another mode.
   *   True if we are just exiting the current mode.
   * @returns {Promise}
   */
  disable(modesDisabled = false) {
    const topBar = document.querySelector('#' + this.id + '-top-bar');
    if (topBar) {
      topBar.classList.add('navigation-plus-hidden');
    }
    return Promise.resolve();
  }

  /**
   * Ajax.
   *
   * Wraps Drupal.ajax in order to pass along navigation+ state in query
   * parameters. We do this because the current path e.g. /node/1 may have a
   * navigationMode cookie set for it, but the JS path we are calling does not.
   * Let's pass the mode state along via query parameter.
   *
   * @param settings
   *   Settings for Drupal.ajax.
   *
   * @returns {Drupal.Ajax}
   *   An instance of Drupal.ajax with query parameters appended to the URL.
   */
  static ajax = (settings) => {
    const url = new URL(window.location.origin + settings.url);
    url.searchParams.set('navigationMode', Drupal.NavigationPlus.getCookieValue('navigationMode') ?? 'none');
    settings.url = url.toString();
    return Drupal.ajax(settings);
  }

  static dialog = (settings) => {

    const ajaxConfig = {
      url: settings.url,
      event: 'click',
      progress: {
        type: 'fullscreen',
        message: settings.message ?? Drupal.t('Loading...'),
      },
      dialogType: 'dialog',
      dialog: {
        width: settings.width ?? 600,
        height: 'auto',
        target: 'layout-builder-modal',
        autoResize: true,
        modal: true,
      }
    };

    let ajax = Drupal.NavigationPlus.ModePluginBase.ajax(ajaxConfig);
    ajax.execute();
  }

}

Drupal.NavigationPlus = Drupal.NavigationPlus || {};
Drupal.NavigationPlus.ModePluginBase = ModePluginBase;
