<?php

declare(strict_types=1);

namespace Drupal\navigation_plus\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\navigation_plus\NavigationPlusUi;
use Drupal\edit_plus\ParamConverter\EntityConverter;
use Drupal\Core\TypedData\TypedDataManagerInterface;
use Drupal\navigation_plus\LoadEditablePageResponseTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for Navigation + routes.
 */
final class LoadEditablePage extends ControllerBase {

  use LoadEditablePageResponseTrait;

  public function __construct(
    protected readonly EntityConverter $entityConverter,
    protected readonly TypedDataManagerInterface $typedDataManager,
    protected readonly NavigationPlusUi $navigationPlusUi,
  ) {}

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('edit_plus.param_converter.entity'),
      $container->get('typed_data_manager'),
      $container->get('navigation_plus.ui'),
    );
  }

  /**
   * Load editable page.
   *
   * When the user initially views a node there are no UI attributes in the
   * markup. When the user enables Edit Mode an AJAX call to this controller
   * is made to reload the page with UI attributes in the markup. Tool plugins
   * should check $this->navigationPlusUi->getMode() before adding markup
   * to the page.
   */
  public function __invoke(EntityInterface $entity, string $view_mode) {
    $mode = $this->navigationPlusUi->getMode();
    $build = $this->getEmptyContent($entity, $view_mode);

    if (!$this->navigationPlusUi->isValidViewMode($entity, $view_mode)) {
      $build['#markup'] = $this->t('Edit +: Invalid view mode for @label', ['@label' => $entity->label()]);
    }

    if (empty($build['#markup']) && !$entity->access('update')) {
      $build['#markup'] = $this->t("You don't have permission to edit this.");
    }

    // Lock editing the entity down to one workspace at a time.
    // @see navigation_plus_entity_view_alter for the non-AJAX version.
    if (empty($build['#markup']) && \Drupal::moduleHandler()->moduleExists('workspaces') && $mode === 'edit') {
      $constraints = array_values(array_filter($entity->getTypedData()->getConstraints(), function ($constraint) {
        return $constraint instanceof \Drupal\workspaces\Plugin\Validation\Constraint\EntityWorkspaceConflictConstraint;
      }));

      if (!empty($constraints)) {
        $violations = $this->typedDataManager->getValidator()->validate(
          $entity->getTypedData(),
          $constraints[0]
        );
        if (count($violations)) {
          $build['#markup'] = $violations->get(0)->getMessage();
        }
      }
    }

    if (empty($build['#markup']) && \Drupal::moduleHandler()->moduleExists('edit_plus') && $mode === 'edit') {
      $entity = $this->entityConverter->loadEntityFromTempstore($entity);
    }

    $build = $this->entityTypeManager()->getViewBuilder($entity->getEntityTypeId())->view($entity, $view_mode);

    return $this->getAjaxReplaceResponse($entity, $view_mode, $build);
  }

}
